<?php
/*
 * Plugin Name: WPfomify - LearnDash Add-on
 * Plugin URI: https://wpfomify.com
 * Version: 1.0.0
 * Description: LearnDash Add-on for WPfomify - The Social Proof Marketing plugin for WordPress
 * Author: IdeaBox Creations
 * Author URI: https://ideaboxcreations.com
 * Copyright: (c) 2016 IdeaBox Creations
 * License: GNU General Public License v2.0
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: ibx-wpfomo
 */

define( 'IBX_WPFOMO_LEARNDASH_VER', '1.0.0' );
define( 'IBX_WPFOMO_LEARNDASH_DIR', plugin_dir_path( __FILE__ ) );
define( 'IBX_WPFOMO_LEARNDASH_URL', plugins_url( '/', __FILE__ ) );
define( 'IBX_WPFOMO_LEARNDASH_PATH', plugin_basename( __FILE__ ) );

class IBX_WPFomo_LearnDash {
    /**
     * Holds the class object.
     *
     * @since 1.0.0
     * @var object
     */
    public static $instance;

    /**
     * Primary class constructor
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        add_action( 'plugins_loaded', array( $this, 'init_hooks' ) );
        add_action( 'admin_init', array( $this, 'plugin_updater' ), 0 );
    }

    /**
     * Initialize the hooks.
     *
     * @since 1.0.0
     * @return void
     */
    public function init_hooks()
    {
        if ( ! class_exists( 'SFWD_LMS' ) ) {
            IBX_WPFomo_Admin::$errors[] = esc_html__( 'Please install and activate LearnDash plugin in order to use LearnDash add-on with WPfomify.', 'ibx-wpfomo' );
            return;
        }

        require_once IBX_WPFOMO_LEARNDASH_DIR . 'classes/class-ibx-wpfomo-learndash.php';

        add_action( 'ibx_wpfomo_before_metabox_load', array( $this, 'init_fields' ) );
        add_filter( 'ibx_wpfomo_conversion_data', array( $this, 'add_conversion_data' ), 10, 2 );
    }

    /**
     * Plugin updater.
     *
     * @since 1.0.0
     */
    public function plugin_updater() {

        if ( ! class_exists( 'IBX_WPFomo_Plugin_Updater' ) ) {
            return;
        }

    	$license_key = trim( get_option( 'ibx_wpfomo_license_key' ) );

    	// setup the updater
    	$updater = new IBX_WPFomo_Plugin_Updater( IBX_WPFOMO_SL_URL, __FILE__, array(
                'version' 	=> IBX_WPFOMO_WOOCOMMERCE_VER, // current version number
                'license' 	=> $license_key, // license key
                'item_id'   => 52017, // id of this addon
    			'item_name' => 'LearnDash Addon', 	// name of this addon
    			'author' 	=> 'IdeaBox Creations',  // author of this addon
    			'beta'		=> false
    		)
    	);
    }

    /**
     * Init fields for LearnDash integration.
     *
     * @since 1.0.0
     * @return void
     */
    public function init_fields()
    {
        add_filter( 'ibx_wpfomo_field_fomo_type', array( $this, 'hide_fields' ), 10, 1 );
        add_filter( 'ibx_wpfomo_field_conversions_source', array( $this, 'conversion_source' ), 10, 1 );
        add_filter( 'ibx_wpfomo_metabox_fields', array( $this, 'add_fields' ), 10, 1 );
    }

    /**
     * Conversion fields.
     *
     * @since 1.0.0
     * @return array
     */
    public function get_fields()
    {
        $fields = array(
            'ld_template'  => array(
                'type'          => 'textarea',
                'label'         => __('Notification Template', 'ibx-wpfomo'),
                'rows'          => 3,
                'default'       => __('{{name}} enrolled for {{title}} {{time}}', 'ibx-wpfomo'),
                'help'          => __('Variables: {{name}}, {{title}}, {{time}}', 'ibx-wpfomo'),
                'sanitize'      => false
            ),
            'ld_course_img'  => array(
                'type'              => 'checkbox',
                'label'             => __('Display Course Image?', 'ibx-wpfomo'),
                'default'           => '1',
                'description'       => __('If enabled, this will override the image settings defined under Config tab -> Image Options.', 'ibx-wpfomo')
            ),
            'ld_course_orders'    => array(
                'type'      => 'select',
                'label'     => __('Display Entrolments of', 'ibx-wpfomo'),
                'default'   => '',
                'options'   => IBX_WPFomo_LearnDash_Helper::get_courses_list()
            ),
            'ld_orders'    => array(
                'type'      => 'number',
                'label'     => __('Number of Enrolments', 'ibx-wpfomo'),
                'default'   => 5,
                'help'      => __('Notifications will be displayed from these enrolments.', 'ibx-wpfomo')
            ),
            'ld_course_link'  => array(
                'type'      => 'select',
                'label'     => __('Link Notification to', 'ibx-wpfomo'),
                'default'   => 'none',
                'options'   => array(
                    'none'      => __('None', 'ibx-wpfomo'),
                    'product'   => __('Course Page', 'ibx-wpfomo'),
                    'custom'    => __('Custom URL', 'ibx-wpfomo')
                ),
                'help'      => __('You can link notification with course page or custom URL.', 'ibx-wpfomo'),
                'toggle'    => array(
                    'custom'    => array(
                        'fields'    => array('ld_custom_url')
                    )
                )
            ),
            'ld_custom_url'    => array(
                'type'      => 'text',
                'label'     => __('Custom URL', 'ibx-wpfomo'),
                'default'   => '',
            )
        );

        return $fields;
    }

    /**
     * Hide fields when reviews is selected.
     *
     * @since 1.0.0
     * @param array $data
     * @return array
     */
    public function hide_fields( $data )
    {
        $fields = $this->get_fields();

        // Hide fields from other field types.
        foreach ( $data['options'] as $option_key => $option_val ) {
            if ( $option_key != 'conversion' ) {
                foreach ( $fields as $key => $field ) {
                    $data['hide'][$option_key]['fields'][] = $key;
                }
            }
        }

        return $data;
    }

    /**
     * Add LearnDash as conversion source.
     *
     * @since 1.0.0
     * @param array $data
     * @return array
     */
    public function conversion_source( $data )
    {
        $data['options']['learndash'] = 'LearnDash';
        $data['toggle']['learndash']['fields'] = array_keys( $this->get_fields() );
        $data['hide']['custom']['fields'] = array('ld_custom_url');

        return $data;
    }

    /**
     * Add fields for LearnDash conversion.
     *
     * @since 1.0.0
     * @param array $data
     * @return array
     */
    public function add_fields( $data )
    {
        $fields = $this->get_fields();

        foreach ( $fields as $key => $field ) {
            $data['content']['sections']['content_section']['fields'][$key] = $field;
        }

        return $data;
    }

    /**
     * Add conversion content for LearnDash.
     *
     * @since 1.0.0
     * @param array $data
     * @param object $settings
     * @return array
     */
    public function add_conversion_data( $data, $settings )
    {
        if ( 'learndash' != $settings->conversions_source ) {
            return $data;
        }

        $transient_key = 'ibx_wpfomo_learndash_orders_' . $settings->post_id;

        $orders = get_transient( $transient_key );

        if ( ! $orders || empty( $orders ) ) {
            if ( '' != $settings->ld_course_orders ) {
                $orders = IBX_WPFomo_LearnDash_Helper::get_orders_by_course( $settings->ld_course_orders );
            } else {
                $orders = IBX_WPFomo_LearnDash_Helper::get_orders( $settings->ld_orders );
            }

            if ( $orders && ! empty( $orders ) ) {
                // Store data in transient for 6 hours.
                set_transient( $transient_key, $orders, 6 * HOUR_IN_SECONDS );
            } else {
                return $data;
            }
        }

        $count = 1;

        foreach ( $orders as $order ) {

            // Limit the number of Orders to be displayed.
            if ( $count == $settings->ld_orders ) {
                break;
            }

            $user_id    = $order->user_id;
            $user       = get_userdata( $user_id );
            $username   = __('Someone', 'ibx-wpfomo');
            $useremail  = $user->user_email;

            if ( ! empty( $user->first_name ) ) {
                $username = $user->first_name;
            }
            if ( ! empty( $user->last_name ) ) {
                $lastname = $user->last_name;
                $username .= ' ' . $lastname[0] . '.';
            }

            $fields_data = array(
                'title'     => get_the_title( $order->post_id ),
                'name'      => $username,
                'email'     => $useremail
            );

            if ( ! empty( $order->activity_updated ) && $order->activity_updated ) {
                $time = human_time_diff( $order->activity_updated, time() );

                ob_start();
                ?>
                <small><?php echo esc_html__( 'About', 'ibx-wpfomo' ) . ' ' . esc_html( $time ) . ' ' . esc_html__( 'ago', 'ibx-wpfomo' ) ?></small>
                <?php
                $time_ago = ob_get_clean();

                $fields_data['time'] = $time_ago;
            }

            // Course URL or Custom URL.
            if ( 'course' == $settings->ld_course_link ) {
                $fields_data['url'] = get_permalink( $order->post_id );
            }
            if ( 'custom' == $settings->ld_course_link ) {
                $fields_data['url'] = esc_url( $settings->ld_custom_url );
            }

            // Course image.
            $course_image = wp_get_attachment_image_src( get_post_thumbnail_id( $order->post_id ), 'thumbnail' );

            if ( ! empty( $course_image ) ) {
                $fields_data['image'] = array(
                    'url'   => $course_image[0]
                );
            }

            $data['fields'][] = $fields_data;

            $count++;
        }

        $data['template'] = $settings->ld_template;

        return $data;
    }

    /**
	 * Returns the singleton instance of the class.
	 *
	 * @since 1.0.0
	 * @return object The IBX_WPFomo_LearnDash object.
	 */
	public static function get_instance()
	{
		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof IBX_WPFomo_LearnDash ) ) {
			self::$instance = new IBX_WPFomo_LearnDash();
		}

		return self::$instance;
	}
}

$ibx_wpfomo_learndash = IBX_WPFomo_LearnDash::get_instance();
